%{
/*
expresso.y - parser for Java, compile with Yacc or Bison
Copyright  (C) 2000 Hugues Johnson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software 
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

union{ int intval; char *identname; }

%}

/*Tokens*/
%token ABSTRACT
%token BOOLEAN
%token BREAK
%token BYTE
%token CASE
%token CATCH
%token CHAR
%token CLASS
%token CONST
%token CONTINUE
%token DEFAULT
%token DO
%token DOUBLE
%token ELSE
%token EXTENDS
%token FALSE
%token FINAL
%token FINALLY
%token FLOAT
%token FOR
%token GOTO
%token IF
%token IMPLEMENTS
%token IMPORT
%token INSTANCEOF
%token INT
%token INTERFACE
%token LONG
%token NATIVE
%token NEW
%token NULLTOKEN
%token PACKAGE
%token PRIVATE
%token PROTECTED
%token PUBLIC
%token RETURN
%token SHORT
%token STATIC
%token SUPER
%token SWITCH
%token SYNCHRONIZED
%token THIS
%token THROW
%token THROWS
%token TRANSIENT
%token TRUE
%token TRY
%token VOID
%token VOLATILE
%token WHILE
%token LPAREN
%token RPAREN
%token LBRACE
%token RBRACE
%token LSQUARE
%token RSQUARE
%token SEMICOLON
%token COMMA
%token PERIOD
%token ASSIGNMENT
%token GREATERTHAN
%token LESSTHAN
%token UNARYNOT
%token TILDE
%token QUESTION
%token COLON
%token EQUAL
%token LESSTHANEQUAL
%token GREATERTHANEQUAL
%token NOTEQUAL
%token OR
%token AND
%token INCREMENT
%token DECREMENT
%token PLUS
%token MINUS
%token MULTIPLY
%token DIVIDE
%token CONCATENATE
%token SINGLEBAR
%token CARROT
%token MODULUS
%token LEFTSHIFT
%token RIGHTSHIFT
%token RIGHTSHIFTZERO
%token PLUSASSIGN
%token MINUSASSIGN
%token MULTIPLYASSIGN
%token DIVIDEASSIGN
%token CONCATENATEASSIGN
%token SINGLEBARASSIGN
%token CARROTASSIGN
%token MODULUSASSIGN
%token LEFTSHIFTASSIGN
%token RIGHTSHIFTASSIGN
%token RIGHTSHIFTZEROASSIGN

/* %token <intval> LITERAL
%token <identname> IDENTIFIER */

%token LITERAL
%token IDENTIFIER

%% /* BNF Grammar */
java_program:
        CLASS IDENTIFIER PUBLIC STATIC VOID IDENTIFIER LPAREN RPAREN LBRACE method_body RBRACE
        ;
method_body:
        block |
        SEMICOLON
        ;
block:
        LBRACE block_statement RBRACE
        ;
block_statement:
        type vardeclaratorlist SEMICOLON |
        statement
        ;
vardeclaratorlist:
        vardeclarator |
        vardeclarator COMMA vardeclaratorlist
        ;
vardeclarator:
        IDENTIFIER |
        IDENTIFIER ASSIGNMENT add_expr
        ;
type:
        INT
        ;
statement:
        block |
        empty_statement |
        if_statement |
        while_statement |
        statement_expression SEMICOLON
        ;
empty_statement:
        SEMICOLON
        ;
if_statement:
        IF LPAREN cond_expr RPAREN then_statement
        ;
then_statement:
        block |
        empty_statement |
        statement_expression SEMICOLON
        ;
while_statement:
        WHILE LPAREN cond_expr RPAREN
        ;
statement_expression:
        assignment |
        cond_expr
        ;
assignment:
        lvalue assignment_op assignment |
        add_expr
        ;
lvalue:
        IDENTIFIER
        ;
assignment_op:
        ASSIGNMENT |
        PLUSASSIGN
        ;
cond_expr:
        add_expr |
        add_expr EQUAL add_expr
        ;
add_expr:
        mult_expr |
        mult_expr PLUS add_expr |
        mult_expr MINUS add_expr
        ;
mult_expr:
        unary_expr |
        unary_expr MULTIPLY mult_expr |
        unary_expr DIVIDE mult_expr
        ;
unary_expr:
        PLUS unary_expr |
        MINUS unary_expr |
        primary_expr
        ;
primary_expr:
        IDENTIFIER |
        LITERAL
        ;
%%

